$NOLIST
$NOSB
$NOMOD51
$INCLUDE (REG152.ASM)
$LIST


;*********************************************************************
;THIS PROGRAM IS WRITTEN TO START A TRANSMISSION IN SDLC MODE UNDER 
;CPU CONTROL AND ASSUMES ALL TRANSMISSIONS WILL BE VALID, THUS THE NAME 
;Transmit Valid Under Cpu Control (TVUCC1).  
;
;REQUIREMENTS:
;           PACKET LENGTH MUST BE AT LEAST ONE BYTE
;
;THE SOURCE OF DATA TO BE TRANSMITTED IS IN EXTERNAL RAM 
;THE STARTING ADDRESS OF THE DATA IS DEFINED BY:
;           MSG_SOURCE (CAN BE UPTO 16 BITS)
;
;THE LENGTH OF OF THE PACKET TO BE TRANSMITTED IS DEFINED BY:
;           MSG_LENGTH (CAN BE UPTO 16 BITS)
;
;INTERNAL RAM LOCATIONS USED AND MODIFIED:
;           7FH     HIGH_MSG_COUNTER
;           7EH     LOW_MSG_COUNTER
;           7DH     HIGH_MSG_POINTER
;           7CH     LOW_MSG_POINTER
;
;SFRS USED AND MODIFIED:
;           ACC
;           BAUD
;           DPL
;           DPH
;           GMOD
;           IE
;           IEN1
;           IFS
;           TFIFO
;           TSTAT
;*********************************************************************

$EJ
;*********************************************************************
;THIS PROGRAM CAN BE BROKEN INTO 4 SEPARATE SECTIONS.  THESE ARE: THE
;INTERRUPT VECTORS (LOCATIONS 0 TO 0FFH), GSC INTERRUPT SERVICE
;SOFTWARE (LOCATIONS 100H TO 1FFH), GSC INITIALIZATION SOFTWARE 
;(LOCATIONS 200H TO 2FFH), AND THOSE SECTIONS OF CODE NEEDED TO MAKE 
;THE PROGRAM FUNCTION (LOCATIONS 300H TO 3FFH).  THIS SHOULD ALLOW A 
;USER TO PULL OUT ONLY THE SECTIONS THAT ARE NEEDED TO COMPLETE A 
;SPECIFIC APPLICATION.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN UNDER THE ASSUMPTION THAT NO TRANSMIT ERRORS 
;WILL EVER OCCUR, HOWEVER, TO KEEP THE PROGRAM FROM HANGING UP IF AN 
;ERROR DOES OCCUR, SECTIONS OF CODE WERE ADDED.  THESE SECTIONS OF CODE 
;ARE IDENTIFIED BY THE PHRASE "EXTRA CODE FOR ERROR CONDITION" BUT THESE 
;SHOULD NOT BE NECESSARY.  ALL REFERENCES TO ANY TYPE OF ERROR(S) APPLY
;TO THOSE ERRORS THAT ARE DETECTED AND FLAGGED BY THE GSC HARDWARE.  
;SDLC WAS CHOSEN AND WHEN THE GSC IS SERVICED BY THE CPU, THERE ARE NO 
;ERRORS THAT CAN BE DETECTED BY HARDWARE.
;*********************************************************************
;ALL OF THE CODE REQUIRED FOR THE ERROR WORK-AROUND OR JUST TO MAKE THE 
;PROGRAM FLOW PROPERLY IS LOCATED ABOVE LOCATION 300H.  ALL OF THE CODE 
;NEEDED TO SERVICE THE GSC IS LOCATED BETWEEN LOCATIONS 100H TO 200H 
;(EXCEPT FOR THE INTERRUPT VECTORS THEMSELVES).  THE CODE USED TO 
;INITIALIZE THE GSC AND TO BEGIN A TRANSMISSION IS IN LOCATIONS 200H TO 
;300H.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN TO GO ALONG WITH THE INTEL FAE TRAINING
;MATERIAL FOR THE 8XC152 INTERMEDIATE TRAINING (JAN 1987) BUT HOPEFULLY
;IS SIMPLE ENOUGH FOR USERS WHO HAVE NOT TAKEN THE TRAINING COURSE TO 
;UNDERSTAND. THE LETTERS IN PARENTHESES AND FOLLOWED BY A HYPHEN, (A) -, 
;REFER TO THE LETTERS IN FLOW CHARTS GIVEN DURING THE FAE TRAINING
;*********************************************************************

$EJ
HIGH_MSG_COUNTER    DATA    7FH
LOW_MSG_COUNTER     DATA    (HIGH_MSG_COUNTER - 1)

HIGH_MSG_POINTER    DATA    (LOW_MSG_COUNTER - 1)
LOW_MSG_POINTER     DATA    (HIGH_MSG_POINTER - 1)

MSG_SOURCE          EQU 100H        ;STARTING LOCATION OF DATA
MSG_LENGTH          EQU 10H         ;LENGTH OF PACKET TO TRANSMIT

ORG 0
    JMP INIT1

ORG 43H
XMIT_VALID:
    JMP VALID_XMIT

ORG 4BH
XMIT_ERROR:
;*********************************************************************
; EXTRA CODE FOR ERROR CONDITION
;*********************************************************************
    JMP ERROR_XMIT

$EJ
ORG 100H
END_OF_MESSAGE?:
;*********************************************************************
; (A,C) - DECIDE IF END OF MESSAGE AND DECREMENT COUNTER
;*********************************************************************
    DJNZ LOW_MSG_COUNTER,CONTINUE_XMIT
    DJNZ HIGH_MSG_COUNTER,CONTINUE_XMIT
                                    ;CHECK FOR END OF MESSAGE

;*********************************************************************
; (E) - DISABLE TRANSMIT VALID INTERRUPT
;*********************************************************************
    CLR EGSTV                       ;CLEAR INTERRUPT

;*********************************************************************
; (G) - REINITIALIZE MESSAGE POINTERS AND COUNTERS
;*********************************************************************
    MOV LOW_MSG_POINTER,#LOW (MSG_SOURCE)
    MOV HIGH_MSG_POINTER,#HIGH (MSG_SOURCE)
                                    ;RE-INITIALIZE POINTER
    JMP END_MESSAGE_SERVICE

CONTINUE_XMIT:
;*********************************************************************
; (B) - LOAD BYTE
;*********************************************************************
    MOV DPL,LOW_MSG_POINTER
    MOV DPH,HIGH_MSG_POINTER
    MOVX A,@DPTR
    MOV TFIFO,A                     ;MOVE NEXT BYTE INTO TFIFO

;*********************************************************************
; (C) - UPDATE POINTERS
;*********************************************************************
    INC DPTR
    MOV LOW_MSG_POINTER,DPL
    MOV HIGH_MSG_POINTER,DPH        ;INCREMENT POINTER AND ALLOWS
                                    ;TFNF TO BE UPDATED

;*********************************************************************
; (D) - DECIDE IF TFIFO IS FULL (IF NO, JUMP TO VALID_XMIT, IF YES,
;       CONTINUE TO VALID_END)
;*********************************************************************
    JB TFNF,END_OF_MESSAGE?         ;CHECK IF TFIFO FULL

END_MESSAGE_SERVICE:
;*********************************************************************
; (H) - RETURN TO MAIN PROGRAM LOOP
;*********************************************************************
    RETI

$EJ
ORG 200H
;*********************************************************************
; USER GENERIC GSC INITIALIZATION
;*********************************************************************
INIT2:
    MOV BAUD,#60                    ;INIT BAUD RATE
    MOV GMOD,#01                    ;INIT FOR SDLC, 8-BIT ADDR
    MOV IFS,#80H                    ;IFS IS USED IN SDLC MODE

    MOV LOW_MSG_POINTER,#LOW (MSG_SOURCE)
    MOV HIGH_MSG_POINTER,#HIGH (MSG_SOURCE)
                                    ;INIT POINTERS TO START AT ADDRESS
                                    ;RESERVED FOR DATA TO BE TRANSMITTED

    MOV LOW_MSG_COUNTER,#0
    MOV HIGH_MSG_COUNTER,#0         ;INIT COUNTERS TO 0 
    RET

;*********************************************************************
;DECIDE IF A TRANSMISSION IS WANTED
;*********************************************************************
TRANSMISSION_DECISION:
    CJNE A,#02,DO_NOT_TRANSMIT      ;TRANSMIT IF ACC=02

;*********************************************************************
; DECIDE IF TRANSMISSION IN PROGRESS
;*********************************************************************
;   MOV A,HIGH_MSG_COUNTER
;   JNZ DO_NOT_TRANSMIT
;   MOV A,LOW_MSG_COUNTER
;   JNZ DO_NOT_TRANSMIT 

;*********************************************************************
;IN FAE TRAINING IT WAS POINTED OUT THAT TO DETERMINE IF A TRANSMISSION
;WAS IN PROGESS ALL THAT WAS NEEDED WAS TO CHECK THE MESSAGE COUNTER.
;IT WAS LATER FOUND THAT BY ONLY CHECKING THE COUNTER IT WAS POSSIBLE
;THAT A NEW TRANSMISSION MIGHT BE LOADED BEFORE TFIFO WAS EMPTIED AND
;THE TWO MESSAGES WOULD BE COMBINED.  TO SOLVE THIS PROBLEM, IT WAS
;NECESSARY TO CHECK THE TEN BIT AND DOING THIS MAKES CHECKING THE COUNTER
;A REDUNDANT TASK.
;*********************************************************************
    JB TEN, DO_NOT_TRANSMIT

;*********************************************************************
; (G) - REINITIALIZE COUNTERS
;*********************************************************************
;THIS WAS MOVED FROM THE END-OF-MESSAGE ROUTINE (GSC TRANSMIT VALID 
;RESPONSE) BECAUSE THE LENGTH OF THE PACKET MAY NOT BE KNOWN UNTIL 
;TRANSMISSION IS ABOUT TO START.  INSTEAD, INITIALIZATION OF THE COUNTERS 
;WAS PLACED HERE, IN THE START-OF-MESSAGE ROUTINE.  THIS ALLOWS THE USER 
;TO CONFIGURE THE PACKET LENGTH WHEN TRANSMISSION STARTS, WHEN THE NUMBER 
;OF BYTES WILL BE KNOWN.
;*********************************************************************
;IN THIS EXAMPLE, AN IMMEDIATED VALUE IS LOADED INTO THE MESSAGE COUNTER
;BECAUSE A FIXED PACKET LENGTH IS BEING USED.  IF VARIABLE SIZED PACKETS
;ARE USED THE CURRENT PACKET LENGTH NEEDS TO BE SPLIT INTO TWO SEPARATE 
;BYTE QUANTITIES.  EACH BYTE SHOULD THEN BE INCREMENTED BY 1.  IF EITHER 
;BYTE SHOULD HAPPEN TO INCREMENT FROM 0FFH TO 0 THE OVERFLOW DOES NOT 
;AFFECT THE PROGRAM AND 0 SHOULD BE LOADED INTO THE APPROPRIATE BYTE.
;*********************************************************************
    MOV HIGH_MSG_COUNTER,#HIGH (MSG_LENGTH) + 1
    MOV LOW_MSG_COUNTER,#LOW (MSG_LENGTH) + 1

;*********************************************************************
; ENABLE TRANSMITTER
;*********************************************************************
    SETB TEN

;*********************************************************************
; ENABLE INTERRUPTS
;*********************************************************************
    SETB EA
    SETB EGSTE                      ;ENABLE ERROR INTERRUPT TO PREVENT
                                    ;PROGRAM FROM HANGING UP
    SETB EGSTV                      ;ENABLE TRANSMIT VALID INTERRUPT
    JMP MAIN

DO_NOT_TRANSMIT:
    JMP MAIN

$EJ
ORG 300H
;*********************************************************************
;START OF EXTRA CODE TO MAKE PROGRAM WORK
;*********************************************************************

VALID_XMIT:
    CALL END_OF_MESSAGE?            ;EGSTV SERVICE ROUTINE

VALID_END:
    RET                             ;RETURN TO MAIN PROGRAM LOOP 


ERROR_XMIT:

;***** EXTRA CODE FOR ERROR CONDITION *****

;THIS SECTION SHOULD NOT BE USED BUT INITIALIZES MOST OF THE SFRS USED
;TO 0 IF AN ERROR OCCURS. THIS SECTION HAS NOT BEEN VERIFIED TO WORK
;PROPERLY.

    MOV GMOD,#0
    MOV BAUD,#0
    MOV IFS,#0
    MOV IE,#0
    MOV IEN1,#0
    MOV TSTAT,#0
    CALL IRET
    JMP INIT1                       ;START PROGRAM AGAIN

IRET:
    RETI                            ;CLEAR INT IN PROG FLAG

INIT1:
;*********************************************************************
; USER GENERIC INITIALIZATION - USER WOULD PLACE HIS INITIALIZATION
; ROUTINES HERE
;*********************************************************************

    CALL INIT2                      ;GSC INITIALIZATION

MAIN:
;*********************************************************************
; USER MAIN PROGRAM LOOP
;*********************************************************************

;*********************************************************************
; LOAD TRANSMISSION DECISION CRITERIA INTO ACCUMULATOR
;*********************************************************************
    MOV A,PRBS                      ;OBTAIN A RANDOM #
    ANL A,#0FH                      ;MASK OFF UPPER NIBBLE
    JMP TRANSMISSION_DECISION

END
