$NOLIST
$NOSB
$NOMOD51
$INCLUDE (REG152.ASM)
$LIST

;*********************************************************************
;THIS PROGRAM IS WRITTEN TO START A TRANSMISSION IN CSMA/CD DETERMINISTIC
;MODE UNDER CPU CONTROL AND ASSUMES ALL TRANSMISSIONS WILL BE VALID, THUS 
;THE NAME Transmit Valid Under Cpu Control #3 (TVUCC3).
;
;REQUIREMENTS:
;           PACKET LENGTH MUST BE AT LEAST ONE BYTE
;
;THE SOURCE OF DATA TO BE TRANSMITTED IS IN EXTERNAL RAM
;THE STARTING ADDRESS OF THE DATA IS DEFINED BY:
;           MSG_SOURCE (CAN BE UPTO 16 BITS)
;
;THE LENGTH OF OF THE PACKET TO BE TRANSMITTED IS DEFINED BY:
;           MSG_LENGTH (CAN BE UPTO 16 BITS)
;
;MAXIMUM NUMBER OF STATIONS PARTICIPATING IN BACKOFF IS DEFINED BY:
;           MAX_STATIONS (CAN BE UPTO 63)
;
;SLOT NUMBER ASSIGNED TO THIS STATION IS DEFINED BY:
;           SLOT_ASSIGNED (CAN BE UPTO 63)
;
;INTERNAL RAM LOCATIONS USED AND MODIFIED:
;           7FH     HIGH_MSG_COUNTER
;           7EH     LOW_MSG_COUNTER
;           7DH     HIGH_MSG_POINTER
;           7CH     LOW_MSG_POINTER
;
;SFRS USED AND MODIFIED:
;           ACC
;           BAUD
;           DPL
;           DPH
;           GMOD
;           IE
;           IEN1
;           IFS
;           MYSLOT
;           PRBS
;           RSTAT
;           SLOTTM
;           TCDCNT
;           TFIFO
;           TSTAT
;*********************************************************************

$EJ

;*********************************************************************
;THIS PROGRAM CAN BE BROKEN INTO 4 SEPARATE SECTIONS.  THESE ARE: THE
;INTERRUPT VECTORS (LOCATIONS 0 TO 0FFH), GSC INTERRUPT SERVICE
;SOFTWARE (LOCATIONS 100H TO 1FFH), GSC INITIALIZATION SOFTWARE 
;(LOCATIONS 200H TO 2FFH), AND THOSE SECTIONS OF CODE NEEDED TO MAKE 
;THE PROGRAM FUNCTION (LOCATIONS 300H TO 3FFH).  THIS SHOULD ALLOW A 
;USER TO PULL OUT ONLY THE SECTIONS THAT ARE NEEDED TO COMPLETE A 
;SPECIFIC APPLICATION.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN UNDER THE ASSUMPTION THAT NO TRANSMIT ERRORS 
;WILL EVER OCCUR, HOWEVER, TO KEEP THE PROGRAM FROM HANGING UP IF AN 
;ERROR DOES OCCUR, SECTIONS OF CODE WERE ADDED.  THESE SECTIONS OF CODE 
;ARE IDENTIFIED BY THE PHRASE "EXTRA CODE FOR ERROR CONDITION" BUT THESE 
;SHOULD NEVER BE USED.  ALL REFERENCES TO ANY TYPE OF ERROR(S) PERTAIN 
;TO THOSE ERRORS THAT ARE DETECTED AND FLAGGED BY THE GSC HARDWARE.  
;CSMA/CD WAS CHOSEN BUT BECAUSE THERE CAN ALWAYS BE HARDWARE DECTECTABLE 
;ERRORS, THIS PROGRAM WILL NOT STAND-ALONE IN A REAL APPLICATION AND 
;WILL NEED TO BE MODIFIED TO HANDLE THE ERRORS THAT MAY OCCUR.  THE 
;PROGRAM TEUCC3.* WILL PERFORM THESE FUNCTIONS YET IT ALSO MIRRORS THIS 
;PROGRAM AS MUCH AS IS POSSIBLE.
;*********************************************************************
;ALL OF THE CODE REQUIRED FOR THE ERROR CONDITION WORK-AROUND OR JUST TO 
;MAKE THE PROGRAM FLOW PROPERLY IS LOCATED ABOVE LOCATION 300H.  ALL OF 
;THE CODE NEEDED TO SERVICE THE GSC IS LOCATED BETWEEN LOCATIONS 100H TO 
;200H (EXCEPT FOR THE INTERRUPT VECTORS THEMSELVES).  THE CODE USED TO 
;INITIALIZE THE GSC AND TO BEGIN A TRANSMISSION IS IN LOCATIONS 200H TO 
;300H.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN TO GO ALONG WITH THE INTEL FAE TRAINING
;MATERIAL FOR THE 8XC152 INTERMEDIATE TRAINING (JAN 1987) BUT HOPEFULLY
;IS SIMPLE ENOUGH FOR USERS WHO HAVE NOT TAKEN THE TRAINING COURSE TO 
;UNDERSTAND. THE LETTERS IN PARENTHESES AND FOLLOWED BY A HYPHEN, (A) -, 
;REFER TO THE LETTERS IN FLOW CHARTS GIVEN DURING THE FAE TRAINING
;*********************************************************************

$EJ

HIGH_MSG_COUNTER    DATA    07FH
LOW_MSG_COUNTER     DATA    (HIGH_MSG_COUNTER - 1)

HIGH_MSG_POINTER    DATA    (LOW_MSG_COUNTER - 1)
LOW_MSG_POINTER     DATA    (HIGH_MSG_POINTER - 1)

MSG_SOURCE          EQU 100H        ;STARTING LOCATION OF DATA TO XMIT
MSG_LENGTH          EQU 10H         ;LENGTH OF PACKET TO XMIT

MAX_STATIONS        EQU 40          ;NUMBER OF STATIONS PARTICIPATING
                                    ;DURING BACKOFF (MAX 63)

SLOT_ASSIGNED       EQU 10          ;SLOT ASSIGNED TO THIS STATIONS
                                    ;(MAX 63)

MY_SLOT             EQU (SLOT_ASSIGNED) OR 11000000B
                                    ;SELECT DCR AND DC JAM.  THE RESULT
                                    ;WILL BE LOADED INTO MYSLOT

ORG 0
    JMP INIT1

ORG 43H
XMIT_VALID:
    JMP VALID_XMIT

ORG 4BH
XMIT_ERROR:
;*********************************************************************
; EXTRA CODE FOR ERROR CONDITION
;*********************************************************************
    JMP ERROR_XMIT

$EJ
ORG 100H
END_OF_MESSAGE?:
;*********************************************************************
; (A,C) - DECIDE IF END OF MESSAGE AND DECREMENT COUNTERS
;*********************************************************************
    DJNZ LOW_MSG_COUNTER,CONTINUE_XMIT
    DJNZ HIGH_MSG_COUNTER,CONTINUE_XMIT
                                    ;IF BOTH THE HIGH AND LOW COUNTER
                                    ;ARE DECREMENTED TO 0 THEN IT IS
                                    ;THE END OF THE MSG

;*********************************************************************
; MESSAGE HAS ENDED
; (E) - DISABLE TRANSMIT VALID INTERRUPT
;*********************************************************************
    CLR EGSTV                       ;CLEAR INTERRUPT

;*********************************************************************
; (G) - REINITIALIZE MESSAGE POINTERS
;*********************************************************************
    MOV LOW_MSG_POINTER,#LOW (MSG_SOURCE)
    MOV HIGH_MSG_POINTER,#HIGH (MSG_SOURCE)
                                    ;RE-INITIALIZE POINTER

    JMP END_MESSAGE_SERVICE

CONTINUE_XMIT:
;*********************************************************************
; (B) - LOAD BYTE
;*********************************************************************
    MOV DPL,LOW_MSG_POINTER
    MOV DPH,HIGH_MSG_POINTER
    MOVX A,@DPTR
    MOV TFIFO,A                     ;MOVE NEXT BYTE INTO TFIFO

;*********************************************************************
; (C) - UPDATE POINTERS AND COUNTERS
;*********************************************************************
    INC DPTR
    MOV LOW_MSG_POINTER,DPL
    MOV HIGH_MSG_POINTER,DPH        ;INCREMENT POINTER AND ALLOWS
                                    ;TFNF TO BE UPDATED

;*********************************************************************
; (D) - DECIDE IF TFIFO IS FULL (IF NO, JUMP TO VALID_XMIT, IF YES,
;       CONTINUE TO VALID_END)
;*********************************************************************
    JB TFNF,END_OF_MESSAGE?         ;CHECK IF TFIFO FULL

END_MESSAGE_SERVICE:
;*********************************************************************
; (H) - RETURN TO MAIN PROGRAM LOOP
;*********************************************************************
    RETI

$EJ
ORG 200H
;*********************************************************************
; USER GENERIC GSC INITIALIZATION
;*********************************************************************
INIT2:
    MOV BAUD,#60                    ;INIT FOR A BAUD RATE DIVISABLE BY 3
    MOV GMOD,#02H                   ;INIT FOR CSMA/CD,8-BIT ADDR,
                                    ;8-BIT PREAMBLE, NORMAL BACKOFF
    MOV IFS,#80H                    ;DEAD TIME BETWEEN FRAMES
    MOV MYSLOT,#MY_SLOT             ;INIT FOR DC JAM,DCR AND ASSIGNED
                                    ;SLOT
    MOV PRBS,#0FFH                  ;FREEZE PRBS
    MOV SLOTTM,#0C0H                ;NUMBER OF BIT TIMES PER SLOT
    MOV TCDCNT,#MAX_STATIONS        ;LOAD TCDCNT WITH NUMBER OF STATIONS
                                    ;PARTICIPATING IN BACKOFF

    MOV LOW_MSG_POINTER,#LOW (MSG_SOURCE)
    MOV HIGH_MSG_POINTER,#HIGH (MSG_SOURCE)
                                    ;INITIALIZE POINTERS TO START AT
                                    ;LOCATION RESERVED FOR DATA TO BE
                                    ;TRANSMITTED
    MOV LOW_MSG_COUNTER,#0
    MOV HIGH_MSG_COUNTER,#0         ;INITIALIZE COUNTERS TO 0 BECAUSE
                                    ;THE FIRST CHECK MADE TO DETERMINE
                                    ;IF A TRANSMISSION IS IN PROGRESS
                                    ;IS TO MAKE SURE COUNTERS ARE 0
    JB P4.7,GSC_INIT_END            ;ENABLE HBA IF P4.7=0
    SETB HABEN

GSC_INIT_END:
    RET

;*********************************************************************
;DECIDE IF A TRANSMISSION IS WANTED
;*********************************************************************
TRANSMISSION_DECISION:
    CJNE A,#0FH,DO_NOT_TRANSMIT     ;TRANSMIT IF ACC=0FH

;*********************************************************************
; DECIDE IF TRANSMISSION IN PROGRESS
;*********************************************************************
;   MOV A,HIGH_MSG_COUNTER
;   JNZ DO_NOT_TRANSMIT
;   MOV A,LOW_MSG_COUNTER
;   JNZ DO_NOT_TRANSMIT

;*********************************************************************
;IN FAE TRAINING IT WAS POINTED OUT THAT TO DETERMINE IF A TRANSMISSION 
;WAS IN PROGRESS ALL THAT WAS NEEDED WAS TO CHECK THE MESSAGE COUNTER.
;IT WAS LATER FOUND THAT THIS ALONE DOES NOT ALWAYS WORK. THE FOLLOWING 
;SCENARIO DEMONSTRATES WHAT THE PROBLEM IS:
;1)THE LAST BYTE OF A TRANSMISSION IS LOADED INTO TFIFO
;2)THE COUNTER NOW EQUALS 0 YET THERE MAY BE UPTO 3 BYTES IN TFIFO)
;3)ANOTHER NEW TRANSMISSION IS REQUESTED BEFORE TFIFO IS EMPTIED
;4)THE COUNTER IS EXAMINED AND
;5)THE NEW TRANSMISSION IS APPENDED TO THE FIRST TRANSMISSION
;6)THE MESSAGE IS NOW LONGER THAN ORIGINALLY INTENDED AND THE
;   NEW ADDRESS IS PART OF THE DATA FIELD
;THE FOLLOWING INSTRUCTION FIXES THIS PROBLEM BY TESTING THE TEN BIT,
;WHICH MAKES CHECKING THE COUNTERS A REDUNDANT TASK.

    JB TEN,DO_NOT_TRANSMIT
    
;*********************************************************************
; (G) - REINITIALIZE COUNTERS
;*********************************************************************
;THIS WAS MOVED FROM THE END-OF-MESSAGE ROUTINE (GSC TRANSMIT VALID 
;RESPONSE) BECAUSE THE LENGTH OF THE PACKET MAY NOT BE KNOWN UNTIL 
;TRANSMISSION IS ABOUT TO START.  INSTEAD, INITIALIZATION OF THE COUNTERS 
;WAS PLACED HERE, IN THE START-OF-MESSAGE ROUTINE.  THIS ALLOWS THE USER 
;TO CONFIGURE THE PACKET LENGTH WHEN TRANSMISSION STARTS, WHEN THE NUMBER 
;OF BYTES WILL BE KNOWN.
;*********************************************************************
;THIS SECTIONS LOADS AN IMMEDIATE VALUE INTO THE COUNTERS BECAUSE FIXED
;MESSAGE LENGTHS ARE USED.  IF VARIABLE LENGTHS ARE USED THEN THE 
;CALCULATED VALUE NEEDS TO BE BROKEN INTO 8 BIT QUANTITIES AND EACH
;BYTE WILL HAVE TO BE INCREMENTED BY 1.  EACH BYTE WOULD THEN BE LOADED 
;INTO EACH COUNTER INDIVIDUALLY.  IF EITHER BYTE INCREMENTS TO 0 
;(OVERFLOW) IT DOES NOT MATTER AND 0 SHOULD BE LOADED INTO THE 
;APPROPRIATE BYTE(S).
;*********************************************************************

    MOV HIGH_MSG_COUNTER,#HIGH (MSG_LENGTH) + 1
    MOV LOW_MSG_COUNTER,#LOW (MSG_LENGTH) + 1
                                    ;1 HAS TO BE ADDED BECAUSE THE
                                    ;DJNZ INSTRUCTION DECREMENTS BEFORE
                                    ;THE DECISION PROCESS

;*********************************************************************
; ENABLE TRANSMITTER
;*********************************************************************
    
    SETB TEN

;*********************************************************************
; ENABLE INTERRUPTS
;*********************************************************************
    SETB EA
    SETB EGSTE                      ;ENABLE ERROR INTERRUPT TO PREVENT
                                    ;PROGRAM FROM HANGING UP
    SETB EGSTV                      ;ENABLE TRANSMIT VALID INTERRUPT

DO_NOT_TRANSMIT:
    JMP MAIN


$EJ
ORG 300H
;*********************************************************************
;START OF EXTRA CODE TO MAKE PROGRAM WORK
;*********************************************************************
VALID_XMIT:
    CALL END_OF_MESSAGE?            ;EGSTV SERVICE ROUTINE

VALID_END:
    RET                             ;RETURN TO MAIN PROGRAM LOOP 

ERROR_XMIT:

;***** EXTRA CODE FOR ERROR CONDITION *****
;THIS SHOULD NOT BE USED BUT IS PLACED HERE IF A TRANSMIT ERROR SHOULD 
;OCCUR.  EVERY SFR USED IS REINITIALIZED TO 0. THIS ERROR SECTION HAS 
;NOT BEEN TESTED.

    MOV GMOD,#0
    MOV BAUD,#0
    MOV IE,#0
    MOV IEN1,#0
    MOV TSTAT,#0
    MOV IFS,#0
    CALL IRET
    MOV MYSLOT,#0
    MOV TCDCNT,#0
    JMP INIT1                       ;START PROGRAM AGAIN

IRET:
    RETI                            ;CLEAR INT IN PROG FLAG

INIT1:
;*********************************************************************
; USER GENERIC INITIALIZATION -  USER WOULD PLACE HIS INITIALIZATION 
; SOFTWARE HERE
;*********************************************************************

    CALL INIT2                      ;GSC INITIALIZATION

MAIN:
;*********************************************************************
; USER MAIN PROGRAM LOOP
;*********************************************************************

;*********************************************************************
; LOAD TRANSMISSION DECISION CRITERIA INTO ACCUMULATOR
;*********************************************************************
    MOV A,PRBS                      ;OBTAIN A RANDOM #
    ANL A,#0FH                      ;MASK OFF UPPER 2 BITS
    JMP TRANSMISSION_DECISION

END
